import React, { PureComponent } from 'react';
import { View, Platform, WebView } from 'react-native';
import PropTypes from 'prop-types';
import * as jsBuilder from './jsBuilder';
import Button from '../Button/Button';

import getStyles from './ChartWrapper.styles';

const additionalOnPressCode = `
        chart.on('click', function(param) {
            var obj = {
                type: 'event_clicked',
                componentType: param.componentType,
                seriesType: param.seriesType,
                seriesIndex: param.seriesIndex,
                name: param.name,
                dataIndex: param.dataIndex,
                data: param.data,
                dataType: param.dataType,
                value: param.value,
                color: param.color,
            };
            sendData(JSON.stringify(obj));
        });
    `;

class ChartWrapper extends PureComponent {
    static propTypes = {
        /** Styles for the component */
        contextStyles: PropTypes.shape({
            /** Styles for the container */
            container: PropTypes.object,
        }),

        /**
         * Triggered on press of the chart button
         * @param {Object} data
         */
        onPress: PropTypes.func,

        /**
         * Chart configuration
         * @see https://echarts.apache.org
         */
        option: PropTypes.object,

        /** Id used for testing */
        testID: PropTypes.string,
    };

    static defaultProps = {
    };

    constructor(props) {
        super(props);
        this.callbacks = {};
        this.state = {
            selectedData: null,
            selectedName: null,
            selectedValue: null,
        };
    }

    render() {
        const {
            contextStyles,
            onPress,
            option,
            testID,
        } = this.props;
        const {
            selectedData,
            selectedName,
            selectedValue,
        } = this.state;

        const styles = getStyles(contextStyles);

        // Generate container props
        const containerProps = {
            style: styles.container,
        };
        if (testID) { containerProps.testID = `${testID}__container`; }

        // Generate WebView props
        const {
            animation: oAnimation,
            ...restOptions
        } = option;
        const injectedProps = {
            option: {
                ...restOptions,
                animation: selectedName ? false : option.animation,
            },
        };
        if (onPress) { injectedProps.additionalCode = additionalOnPressCode; }
        const source =
            Platform.OS === 'ios'
                ? require('./echarts.html')

                // TODO: update android path???
                : { uri: 'file:///android_asset/echarts.html' };
        const webViewProps = {
            useWebKit: false,
            originWhitelist: [ '*' ],
            scrollEnabled: false,
            source,
            injectedJavaScript: jsBuilder.getJavascriptSource(injectedProps),
            onMessage: this.handleMessage,
            allowFileAccess: true,
            allowUniversalAccessFromFileURLs: true,
            mixedContentMode: 'always',
            ref: this.handleWebViewRef,
            key: JSON.stringify(restOptions),
            style: { backgroundColor: 'transparent' },
        };

        // Generate text props
        const buttonProps = {
            contextStyles: {
                container: styles.button,
            },
            text: `Open details for ${selectedName}: ${selectedValue || ''}`,
            disabled: !onPress || !selectedName,
        };
        if (onPress) { buttonProps.onPress = () => { onPress(selectedData); }; }

        return (
            <View { ...containerProps }>
                <WebView { ...webViewProps } />
                { selectedName && onPress &&
                <Button { ...buttonProps } />
                }
            </View>
        );
    }

    /**
     * Handles data from the web view
     * @param {Object} event
     */
    handleMessage = (event) => {
        try {
            if (event) {
                const { nativeEvent } = event;
                const { data } = nativeEvent;
                const dataObj = JSON.parse(data);
                const {
                    types,
                    payload,
                } = dataObj;

                // Handle event data for press event
                if (types === 'DATA') {
                    const payloadObj = JSON.parse(payload);
                    const {
                        type: payloadType,
                        data: payloadData,
                    } = payloadObj;

                    // Ensure press event type is from additionalOnPressCode
                    if (payloadType === 'event_clicked') {
                        // Pass chart data to press event
                        this.setState({
                            selectedData: payloadData,
                            selectedName: payloadData.name,
                            selectedValue: payloadData.value,
                        });
                    }
                }

                // Handle getOption callback
                else if (dataObj.types === 'CALLBACK') {
                    const { uuid } = dataObj;
                    this.callbacks[uuid](dataObj.payload);
                }
            }
        }
        catch (error) {
            // console.log(error);
        }
    };

    /**
     * Sends data to the web view
     * @param {Object} data
     */
    postMessage = (data) => {
        this.webview.postMessage(jsBuilder.convertToPostMessageString(data));
    };

    /**
     * Generate UUID
     * @returns {String}
     */
    ID = () => (
        `_${Math.random()
            .toString(36)
            .substr(2, 9)}`
    );

    /**
     * Returns the current configuration of the chart instance
     * @param {Function} callback
     * @param {Array<String>} properties
     */
    getOption = (callback, properties = undefined) => {
        const uuid = this.ID();
        this.callbacks[uuid] = callback;
        const data = {
            types: 'GET_OPTION',
            uuid,
            properties,
        };
        this.postMessage(data);
    };

    /**
     * Sets a chart configuration dynamically (after initial setup with option prop)
     * @param {Object} option
     * @param {Boolean} notMerge
     * @param {Boolean} lazyUpdate
     */
    setOption = (option, notMerge, lazyUpdate) => {
        const data = {
            types: 'SET_OPTION',
            payload: {
                option,
                notMerge: notMerge || false,
                lazyUpdate: lazyUpdate || false,
            },
        };
        this.postMessage(data);
    };

    /**
     * Sets ref for web view
     * @param {Object} ref
     */
    handleWebViewRef = (ref) => {
        this.webview = ref;
    };
}

export default ChartWrapper;
